/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: IEjbJarDeployableMetadata.java 4610 2009-02-17 08:49:37Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.api;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.IDeployableMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEJB3;
import org.ow2.util.ee.metadata.ws.api.IWebservicesDeployableMetadata;

/**
 * IEjbJarDeployableMetadata.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Gael Lalire
 */
public interface IEjbJarDeployableMetadata<
   E extends EJBDeployable<E>,
   D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
   C extends IEjbJarClassMetadata<E, D, C, M, F>,
   M extends IEjbJarMethodMetadata<E, D, C, M, F>,
   F extends IEjbJarFieldMetadata<E, D, C, M, F>>
    extends IDeployableMetadata<E>, IWebservicesDeployableMetadata {

    /**
     * Gets the class metadata for the given ejb-name/ejbclass.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     * @param ejbClass the class of the EJB.
     */
    C getBeanEjbJarClassMetadataForEjbName(final String ejbName, final String ejbClass);


    /**
     * Add annotation metadata for a given class.
     * @param classAnnotationMetadata annotation metadata of a class.
     */
    void addScannedClassMetadata(final C classAnnotationMetadata);

    /**
     * Gets the class metadata for the given ejb-name.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     */
    C getEjbJarClassMetadataForEjbName(final String ejbName);

    /**
     * Get interceptor class metadata.
     * @param className key of the map of annotations bean.
     * @return interceptor class metadata of a given name or null if class not
     *         found.
     */
    C getScannedClassMetadata(final String className);

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    Collection<C> getEjbJarClassMetadataCollection();

    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar
     *         metadata.
     */
    Map<String, ApplicationException> getApplicationExceptions();

    /**
     * @return object representing list of default interceptor classes.
     */
    IJInterceptors getDefaultInterceptorsClasses();

    /**
     * Sets the object representing the default interceptor classes.
     * @param defaultInterceptorsClasses list of classes
     */
    void setDefaultInterceptorsClasses(final IJInterceptors defaultInterceptorsClasses);


    /**
     * @return the ejb3 deployment descriptor object.
     */
    IEJB3 getEjb3();

    /**
     * Sets the ejb3 deployment descriptor object.
     * @param ejb3 the ejb3 deployment descriptor object.
     */
    void setEjb3(final IEJB3 ejb3);

    /**
     * @return a list of all beans that are defined
     */
    List<String> getBeanNames();

    /**
     * @param beanName the bean that will be used in order to get the classes.
     * @return a list of all classes that are for a given bean
     */
    Collection<C> getClassesForBean(final String beanName);

}
